@@ -1,6 +1,10 @@
+from .. import cbook
 from .backend_qt5cairo import _BackendQT5Cairo, FigureCanvasQTCairo
 
 
+cbook.warn_deprecated("3.3", name=__name__, obj_type="backend")
+
+
 @_BackendQT5Cairo.export
 class _BackendQT4Cairo(_BackendQT5Cairo):
     class FigureCanvas(FigureCanvasQTCairo):
