@@ -48,7 +48,7 @@ mpl.mpl_figure_comm = function (comm, msg) {
         console.error('Failed to find cell for figure', id, fig);
         return;
     }
-    fig.cell_info[0].output_area.element.one(
+    fig.cell_info[0].output_area.element.on(
         'cleared',
         { fig: fig },
         fig._remove_fig_handler
@@ -61,6 +61,7 @@ mpl.figure.prototype.handle_close = function (fig, msg) {
         'cleared',
         fig._remove_fig_handler
     );
+    fig.resizeObserverInstance.unobserve(fig.canvas_div);
 
     // Update the output cell to use the data from the current canvas.
     fig.push_to_output();
@@ -181,6 +182,10 @@ mpl.figure.prototype._init_toolbar = function () {
 
 mpl.figure.prototype._remove_fig_handler = function (event) {
     var fig = event.data.fig;
+    if (event.target !== this) {
+        // Ignore bubbled events from children.
+        return;
+    }
     fig.close_ws(fig, {});
 };
 
