@@ -2,38 +2,25 @@
 # Created: 23 Sep 2005
 # Parts rewritten by Reinier Heeres <reinier@heeres.eu>
 
-import copy
-
 import numpy as np
 
+import matplotlib.transforms as mtransforms
 from matplotlib import (
-    artist, cbook, lines as mlines, axis as maxis, patches as mpatches,
-    rcParams)
+    artist, lines as mlines, axis as maxis, patches as mpatches, rcParams)
 from . import art3d, proj3d
 
 
-def get_flip_min_max(coord, index, mins, maxs):
-    if coord[index] == mins[index]:
-        return maxs[index]
-    else:
-        return mins[index]
-
-
 def move_from_center(coord, centers, deltas, axmask=(True, True, True)):
-    '''Return a coordinate that is moved by "deltas" away from the center.'''
-    coord = copy.copy(coord)
-    for i in range(3):
-        if not axmask[i]:
-            continue
-        if coord[i] < centers[i]:
-            coord[i] -= deltas[i]
-        else:
-            coord[i] += deltas[i]
-    return coord
+    """
+    For each coordinate where *axmask* is True, move *coord* away from
+    *centers* by *deltas*.
+    """
+    coord = np.asarray(coord)
+    return coord + axmask * np.copysign(1, coord - centers) * deltas
 
 
 def tick_update_position(tick, tickxs, tickys, labelpos):
-    '''Update tick line and label position and style.'''
+    """Update tick line and label position and style."""
 
     tick.label1.set_position(labelpos)
     tick.label2.set_position(labelpos)
@@ -46,7 +33,7 @@ def tick_update_position(tick, tickxs, tickys, labelpos):
 
 
 class Axis(maxis.XAxis):
-    """An Axis class for the 3D plots. """
+    """An Axis class for the 3D plots."""
     # These points from the unit cube make up the x, y and z-planes
     _PLANES = (
         (0, 3, 7, 4), (1, 2, 6, 5),     # yz planes
@@ -57,11 +44,11 @@ class Axis(maxis.XAxis):
     # Some properties for the axes
     _AXINFO = {
         'x': {'i': 0, 'tickdir': 1, 'juggled': (1, 0, 2),
-            'color': (0.95, 0.95, 0.95, 0.5)},
+              'color': (0.95, 0.95, 0.95, 0.5)},
         'y': {'i': 1, 'tickdir': 0, 'juggled': (0, 1, 2),
-            'color': (0.90, 0.90, 0.90, 0.5)},
+              'color': (0.90, 0.90, 0.90, 0.5)},
         'z': {'i': 2, 'tickdir': 0, 'juggled': (0, 2, 1),
-            'color': (0.925, 0.925, 0.925, 0.5)},
+              'color': (0.925, 0.925, 0.925, 0.5)},
     }
 
     def __init__(self, adir, v_intervalx, d_intervalx, axes, *args,
@@ -73,37 +60,48 @@ class Axis(maxis.XAxis):
         # Do not depend on this existing in future releases!
         self._axinfo = self._AXINFO[adir].copy()
         if rcParams['_internal.classic_mode']:
-            self._axinfo.update(
-                {'label': {'va': 'center',
-                           'ha': 'center'},
-                 'tick': {'inward_factor': 0.2,
-                          'outward_factor': 0.1,
-                          'linewidth': rcParams['lines.linewidth'],
-                          'color': 'k'},
-                 'axisline': {'linewidth': 0.75,
-                              'color': (0, 0, 0, 1)},
-                 'grid': {'color': (0.9, 0.9, 0.9, 1),
-                          'linewidth': 1.0,
-                          'linestyle': '-'},
-                 })
+            self._axinfo.update({
+                'label': {'va': 'center', 'ha': 'center'},
+                'tick': {
+                    'inward_factor': 0.2,
+                    'outward_factor': 0.1,
+                    'linewidth': {
+                        True: rcParams['lines.linewidth'],  # major
+                        False: rcParams['lines.linewidth'],  # minor
+                    }
+                },
+                'axisline': {'linewidth': 0.75, 'color': (0, 0, 0, 1)},
+                'grid': {
+                    'color': (0.9, 0.9, 0.9, 1),
+                    'linewidth': 1.0,
+                    'linestyle': '-',
+                },
+            })
         else:
-            self._axinfo.update(
-                {'label': {'va': 'center',
-                           'ha': 'center'},
-                 'tick': {'inward_factor': 0.2,
-                          'outward_factor': 0.1,
-                          'linewidth': rcParams.get(
-                              adir + 'tick.major.width',
-                              rcParams['xtick.major.width']),
-                          'color': rcParams.get(
-                              adir + 'tick.color',
-                              rcParams['xtick.color'])},
-                 'axisline': {'linewidth': rcParams['axes.linewidth'],
-                              'color': rcParams['axes.edgecolor']},
-                 'grid': {'color': rcParams['grid.color'],
-                          'linewidth': rcParams['grid.linewidth'],
-                          'linestyle': rcParams['grid.linestyle']},
-                 })
+            self._axinfo.update({
+                'label': {'va': 'center', 'ha': 'center'},
+                'tick': {
+                    'inward_factor': 0.2,
+                    'outward_factor': 0.1,
+                    'linewidth': {
+                        True: (  # major
+                            rcParams['xtick.major.width'] if adir in 'xz' else
+                            rcParams['ytick.major.width']),
+                        False: (  # minor
+                            rcParams['xtick.minor.width'] if adir in 'xz' else
+                            rcParams['ytick.minor.width']),
+                    }
+                },
+                'axisline': {
+                    'linewidth': rcParams['axes.linewidth'],
+                    'color': rcParams['axes.edgecolor'],
+                },
+                'grid': {
+                    'color': rcParams['grid.color'],
+                    'linewidth': rcParams['grid.linewidth'],
+                    'linestyle': rcParams['grid.linestyle'],
+                },
+            })
 
         maxis.XAxis.__init__(self, axes, *args, **kwargs)
 
@@ -135,20 +133,20 @@ class Axis(maxis.XAxis):
         self.label._transform = self.axes.transData
         self.offsetText._transform = self.axes.transData
 
-    @cbook.deprecated("3.1")
-    def get_tick_positions(self):
-        majorLocs = self.major.locator()
-        majorLabels = self.major.formatter.format_ticks(majorLocs)
-        return majorLabels, majorLocs
-
     def get_major_ticks(self, numticks=None):
         ticks = maxis.XAxis.get_major_ticks(self, numticks)
         for t in ticks:
-            t.tick1line.set_transform(self.axes.transData)
-            t.tick2line.set_transform(self.axes.transData)
-            t.gridline.set_transform(self.axes.transData)
-            t.label1.set_transform(self.axes.transData)
-            t.label2.set_transform(self.axes.transData)
+            for obj in [
+                    t.tick1line, t.tick2line, t.gridline, t.label1, t.label2]:
+                obj.set_transform(self.axes.transData)
+        return ticks
+
+    def get_minor_ticks(self, numticks=None):
+        ticks = maxis.XAxis.get_minor_ticks(self, numticks)
+        for t in ticks:
+            for obj in [
+                    t.tick1line, t.tick2line, t.gridline, t.label1, t.label2]:
+                obj.set_transform(self.axes.transData)
         return ticks
 
     def set_pane_pos(self, xys):
@@ -158,7 +156,7 @@ class Axis(maxis.XAxis):
         self.stale = True
 
     def set_pane_color(self, color):
-        '''Set pane color to a RGBA tuple.'''
+        """Set pane color to a RGBA tuple."""
         self._axinfo['color'] = color
         self.pane.set_edgecolor(color)
         self.pane.set_facecolor(color)
@@ -166,10 +164,10 @@ class Axis(maxis.XAxis):
         self.stale = True
 
     def set_rotate_label(self, val):
-        '''
+        """
         Whether to rotate the axis label: True, False or None.
         If set to None the label will be rotated if longer than 4 chars.
-        '''
+        """
         self._rotate_label = val
         self.stale = True
 
@@ -180,15 +178,11 @@ class Axis(maxis.XAxis):
             return len(text) > 4
 
     def _get_coord_info(self, renderer):
-        minx, maxx, miny, maxy, minz, maxz = self.axes.get_w_lims()
-        if minx > maxx:
-            minx, maxx = maxx, minx
-        if miny > maxy:
-            miny, maxy = maxy, miny
-        if minz > maxz:
-            minz, maxz = maxz, minz
-        mins = np.array((minx, miny, minz))
-        maxs = np.array((maxx, maxy, maxz))
+        mins, maxs = np.array([
+            self.axes.get_xbound(),
+            self.axes.get_ybound(),
+            self.axes.get_zbound(),
+        ]).T
         centers = (maxs + mins) / 2.
         deltas = (maxs - mins) / 12.
         mins = mins - deltas / 4.
@@ -203,7 +197,7 @@ class Axis(maxis.XAxis):
         return mins, maxs, centers, deltas, tc, highs
 
     def draw_pane(self, renderer):
-        renderer.open_group('pane3d')
+        renderer.open_group('pane3d', gid=self.get_gid())
 
         mins, maxs, centers, deltas, tc, highs = self._get_coord_info(renderer)
 
@@ -222,7 +216,7 @@ class Axis(maxis.XAxis):
     @artist.allow_rasterization
     def draw(self, renderer):
         self.label._transform = self.axes.transData
-        renderer.open_group('axis3d')
+        renderer.open_group('axis3d', gid=self.get_gid())
 
         ticks = self._update_ticks()
 
@@ -233,29 +227,26 @@ class Axis(maxis.XAxis):
 
         # Determine grid lines
         minmax = np.where(highs, maxs, mins)
+        maxmin = np.where(highs, mins, maxs)
 
         # Draw main axis line
         juggled = info['juggled']
         edgep1 = minmax.copy()
-        edgep1[juggled[0]] = get_flip_min_max(edgep1, juggled[0], mins, maxs)
+        edgep1[juggled[0]] = maxmin[juggled[0]]
 
         edgep2 = edgep1.copy()
-        edgep2[juggled[1]] = get_flip_min_max(edgep2, juggled[1], mins, maxs)
-        pep = proj3d.proj_trans_points([edgep1, edgep2], renderer.M)
-        centpt = proj3d.proj_transform(
-            centers[0], centers[1], centers[2], renderer.M)
-        self.line.set_data((pep[0][0], pep[0][1]), (pep[1][0], pep[1][1]))
+        edgep2[juggled[1]] = maxmin[juggled[1]]
+        pep = np.asarray(
+            proj3d.proj_trans_points([edgep1, edgep2], renderer.M))
+        centpt = proj3d.proj_transform(*centers, renderer.M)
+        self.line.set_data(pep[0], pep[1])
         self.line.draw(renderer)
 
         # Grid points where the planes meet
-        xyz0 = []
-        for tick in ticks:
-            coord = minmax.copy()
-            coord[index] = tick.get_loc()
-            xyz0.append(coord)
+        xyz0 = np.tile(minmax, (len(ticks), 1))
+        xyz0[:, index] = [tick.get_loc() for tick in ticks]
 
         # Draw labels
-        peparray = np.asanyarray(pep)
         # The transAxes transform is used because the Text object
         # rotates the text relative to the display coordinate system.
         # Therefore, if we want the labels to remain parallel to the
@@ -263,10 +254,10 @@ class Axis(maxis.XAxis):
         # edge points of the plane to display coordinates and calculate
         # an angle from that.
         # TODO: Maybe Text objects should handle this themselves?
-        dx, dy = (self.axes.transAxes.transform([peparray[0:2, 1]]) -
-                  self.axes.transAxes.transform([peparray[0:2, 0]]))[0]
+        dx, dy = (self.axes.transAxes.transform([pep[0:2, 1]]) -
+                  self.axes.transAxes.transform([pep[0:2, 0]]))[0]
 
-        lxyz = 0.5*(edgep1 + edgep2)
+        lxyz = 0.5 * (edgep1 + edgep2)
 
         # A rough estimate; points are ambiguous since 3D plots rotate
         ax_scale = self.axes.bbox.size / self.figure.bbox.size
@@ -279,8 +270,7 @@ class Axis(maxis.XAxis):
         axmask = [True, True, True]
         axmask[index] = False
         lxyz = move_from_center(lxyz, centers, labeldeltas, axmask)
-        tlx, tly, tlz = proj3d.proj_transform(lxyz[0], lxyz[1], lxyz[2],
-                                              renderer.M)
+        tlx, tly, tlz = proj3d.proj_transform(*lxyz, renderer.M)
         self.label.set_position((tlx, tly))
         if self.get_rotate_label(self.label.get_text()):
             angle = art3d._norm_text_angle(np.rad2deg(np.arctan2(dy, dx)))
@@ -300,10 +290,8 @@ class Axis(maxis.XAxis):
             outeredgep = edgep2
             outerindex = 1
 
-        pos = copy.copy(outeredgep)
-        pos = move_from_center(pos, centers, labeldeltas, axmask)
-        olx, oly, olz = proj3d.proj_transform(
-            pos[0], pos[1], pos[2], renderer.M)
+        pos = move_from_center(outeredgep, centers, labeldeltas, axmask)
+        olx, oly, olz = proj3d.proj_transform(*pos, renderer.M)
         self.offsetText.set_text(self.major.formatter.get_offset())
         self.offsetText.set_position((olx, oly))
         angle = art3d._norm_text_angle(np.rad2deg(np.arctan2(dy, dx)))
@@ -321,16 +309,16 @@ class Axis(maxis.XAxis):
         # using the wrong reference points).
         #
         # (TT, FF, TF, FT) are the shorthand for the tuple of
-        #   (centpt[info['tickdir']] <= peparray[info['tickdir'], outerindex],
-        #    centpt[index] <= peparray[index, outerindex])
+        #   (centpt[info['tickdir']] <= pep[info['tickdir'], outerindex],
+        #    centpt[index] <= pep[index, outerindex])
         #
         # Three-letters (e.g., TFT, FTT) are short-hand for the array of bools
         # from the variable 'highs'.
         # ---------------------------------------------------------------------
-        if centpt[info['tickdir']] > peparray[info['tickdir'], outerindex]:
+        if centpt[info['tickdir']] > pep[info['tickdir'], outerindex]:
             # if FT and if highs has an even number of Trues
-            if (centpt[index] <= peparray[index, outerindex]
-                    and len(highs.nonzero()[0]) % 2 == 0):
+            if (centpt[index] <= pep[index, outerindex]
+                    and np.count_nonzero(highs) % 2 == 0):
                 # Usually, this means align right, except for the FTT case,
                 # in which offset for axis 1 and 2 are aligned left.
                 if highs.tolist() == [False, True, True] and index in (1, 2):
@@ -342,8 +330,8 @@ class Axis(maxis.XAxis):
                 align = 'left'
         else:
             # if TF and if highs has an even number of Trues
-            if (centpt[index] > peparray[index, outerindex]
-                    and len(highs.nonzero()[0]) % 2 == 0):
+            if (centpt[index] > pep[index, outerindex]
+                    and np.count_nonzero(highs) % 2 == 0):
                 # Usually mean align left, except if it is axis 2
                 if index == 2:
                     align = 'right'
@@ -357,31 +345,19 @@ class Axis(maxis.XAxis):
         self.offsetText.set_ha(align)
         self.offsetText.draw(renderer)
 
-        # Draw grid lines
-        if len(xyz0) > 0:
-            # Grid points at end of one plane
-            xyz1 = copy.deepcopy(xyz0)
-            newindex = (index + 1) % 3
-            newval = get_flip_min_max(xyz1[0], newindex, mins, maxs)
-            for i in range(len(ticks)):
-                xyz1[i][newindex] = newval
-
-            # Grid points at end of the other plane
-            xyz2 = copy.deepcopy(xyz0)
-            newindex = (index + 2) % 3
-            newval = get_flip_min_max(xyz2[0], newindex, mins, maxs)
-            for i in range(len(ticks)):
-                xyz2[i][newindex] = newval
-
-            lines = list(zip(xyz1, xyz0, xyz2))
-            if self.axes._draw_grid:
-                self.gridlines.set_segments(lines)
-                self.gridlines.set_color([info['grid']['color']] * len(lines))
-                self.gridlines.set_linewidth(
-                    [info['grid']['linewidth']] * len(lines))
-                self.gridlines.set_linestyle(
-                    [info['grid']['linestyle']] * len(lines))
-                self.gridlines.draw(renderer, project=True)
+        if self.axes._draw_grid and len(ticks):
+            # Grid lines go from the end of one plane through the plane
+            # intersection (at xyz0) to the end of the other plane.  The first
+            # point (0) differs along dimension index-2 and the last (2) along
+            # dimension index-1.
+            lines = np.stack([xyz0, xyz0, xyz0], axis=1)
+            lines[:, 0, index - 2] = maxmin[index - 2]
+            lines[:, 2, index - 1] = maxmin[index - 1]
+            self.gridlines.set_segments(lines)
+            self.gridlines.set_color(info['grid']['color'])
+            self.gridlines.set_linewidth(info['grid']['linewidth'])
+            self.gridlines.set_linestyle(info['grid']['linestyle'])
+            self.gridlines.draw(renderer, project=True)
 
         # Draw ticks
         tickdir = info['tickdir']
@@ -392,20 +368,17 @@ class Axis(maxis.XAxis):
             ticksign = -1
 
         for tick in ticks:
-
             # Get tick line positions
-            pos = copy.copy(edgep1)
+            pos = edgep1.copy()
             pos[index] = tick.get_loc()
             pos[tickdir] = (
                 edgep1[tickdir]
                 + info['tick']['outward_factor'] * ticksign * tickdelta)
-            x1, y1, z1 = proj3d.proj_transform(pos[0], pos[1], pos[2],
-                                               renderer.M)
+            x1, y1, z1 = proj3d.proj_transform(*pos, renderer.M)
             pos[tickdir] = (
                 edgep1[tickdir]
                 - info['tick']['inward_factor'] * ticksign * tickdelta)
-            x2, y2, z2 = proj3d.proj_transform(pos[0], pos[1], pos[2],
-                                               renderer.M)
+            x2, y2, z2 = proj3d.proj_transform(*pos, renderer.M)
 
             # Get position of label
             default_offset = 8.  # A rough estimate
@@ -416,23 +389,63 @@ class Axis(maxis.XAxis):
             axmask[index] = False
             pos[tickdir] = edgep1[tickdir]
             pos = move_from_center(pos, centers, labeldeltas, axmask)
-            lx, ly, lz = proj3d.proj_transform(pos[0], pos[1], pos[2],
-                                               renderer.M)
+            lx, ly, lz = proj3d.proj_transform(*pos, renderer.M)
 
             tick_update_position(tick, (x1, x2), (y1, y2), (lx, ly))
-            tick.tick1line.set_linewidth(info['tick']['linewidth'])
-            tick.tick1line.set_color(info['tick']['color'])
+            tick.tick1line.set_linewidth(
+                info['tick']['linewidth'][tick._major])
             tick.draw(renderer)
 
         renderer.close_group('axis3d')
         self.stale = False
 
-    # TODO: Get this to work properly when mplot3d supports
-    #       the transforms framework.
-    def get_tightbbox(self, renderer):
-        # Currently returns None so that Axis.get_tightbbox
-        # doesn't return junk info.
-        return None
+    # TODO: Get this to work (more) properly when mplot3d supports the
+    #       transforms framework.
+    def get_tightbbox(self, renderer, *, for_layout_only=False):
+        # inherited docstring
+        if not self.get_visible():
+            return
+        # We have to directly access the internal data structures
+        # (and hope they are up to date) because at draw time we
+        # shift the ticks and their labels around in (x, y) space
+        # based on the projection, the current view port, and their
+        # position in 3D space.  If we extend the transforms framework
+        # into 3D we would not need to do this different book keeping
+        # than we do in the normal axis
+        major_locs = self.get_majorticklocs()
+        minor_locs = self.get_minorticklocs()
+
+        ticks = [*self.get_minor_ticks(len(minor_locs)),
+                 *self.get_major_ticks(len(major_locs))]
+        view_low, view_high = self.get_view_interval()
+        if view_low > view_high:
+            view_low, view_high = view_high, view_low
+        interval_t = self.get_transform().transform([view_low, view_high])
+
+        ticks_to_draw = []
+        for tick in ticks:
+            try:
+                loc_t = self.get_transform().transform(tick.get_loc())
+            except AssertionError:
+                # Transform.transform doesn't allow masked values but
+                # some scales might make them, so we need this try/except.
+                pass
+            else:
+                if mtransforms._interval_contains_close(interval_t, loc_t):
+                    ticks_to_draw.append(tick)
+
+        ticks = ticks_to_draw
+
+        bb_1, bb_2 = self._get_tick_bboxes(ticks, renderer)
+        other = []
+
+        if self.line.get_visible():
+            other.append(self.line.get_window_extent(renderer))
+        if (self.label.get_visible() and not for_layout_only and
+                self.label.get_text()):
+            other.append(self.label.get_window_extent(renderer))
+
+        return mtransforms.Bbox.union([*bb_1, *bb_2, *other])
 
     @property
     def d_interval(self):
@@ -440,7 +453,7 @@ class Axis(maxis.XAxis):
 
     @d_interval.setter
     def d_interval(self, minmax):
-        return self.set_data_interval(*minmax)
+        self.set_data_interval(*minmax)
 
     @property
     def v_interval(self):
@@ -448,7 +461,7 @@ class Axis(maxis.XAxis):
 
     @v_interval.setter
     def v_interval(self, minmax):
-        return self.set_view_interval(*minmax)
+        self.set_view_interval(*minmax)
 
 
 # Use classes to look at different data limits
